;
; $Id: ipfauth.sql 121 2009-03-04 16:51:44Z frey $
;

CREATE TABLE userClass (
  ucId          SERIAL UNIQUE NOT NULL PRIMARY KEY,
  description   CHARACTER VARYING(128),
  rank          INTEGER UNIQUE NOT NULL
);

CREATE TABLE byClassLimits (
  ucId          INTEGER REFERENCES userClass(ucId) ON DELETE CASCADE,
  maxAddresses  INTEGER NOT NULL,
  maxTTL        INTERVAL NOT NULL DEFAULT '1 week'::INTERVAL,
  minRefresh    INTERVAL NOT NULL DEFAULT '1 day'::INTERVAL
);

CREATE TABLE authzAddress (
  emplid        CHARACTER VARYING(11) NOT NULL,
  created       TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT now(),
  expires       TIMESTAMP WITH TIME ZONE NOT NULL,
  remoteAddress INET NOT NULL
);

CREATE INDEX emplidIndex ON authzAddress (emplid);
CREATE INDEX remoteAddressIndex ON authzAddress (remoteAddress);

CREATE FUNCTION expire() RETURNS void AS '
  DELETE FROM authzAddress WHERE expires <= now();
' LANGUAGE SQL;

CREATE FUNCTION expireForEmplId(CHARACTER VARYING(11)) RETURNS void AS '
  DELETE FROM authzAddress WHERE expires <= now() AND emplid = $1;
' LANGUAGE SQL;

CREATE LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION pgipfAuthorize(INET,INTEGER) RETURNS BOOLEAN AS $$
DECLARE
  aRow                RECORD;
  inRemoteAddress     ALIAS FOR $1;
  inLocalPort         ALIAS FOR $2;
BEGIN
  SELECT * INTO aRow FROM authzAddress WHERE
    remoteAddress = inRemoteAddress AND expires > now();
  IF FOUND THEN
    RETURN TRUE;
  END IF;
  RETURN FALSE;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION ipfcacheinvalidate() RETURNS TRIGGER AS '/usr/local/pgipfauth/0.1/src/ipfcacheinvalidate.so' LANGUAGE C;

CREATE TRIGGER ipfcacheinvalidate AFTER INSERT OR UPDATE OR DELETE ON authzAddress FOR EACH ROW EXECUTE PROCEDURE ipfcacheinvalidate();



insert into userclass (description,rank) values ('staff',0);
insert into userclass (description,rank) values ('student',10);

insert into byclasslimits (ucid,maxaddresses,maxttl) values (1,10,'1 month'::interval);
insert into byclasslimits (ucid,maxaddresses,maxttl) values (2,2,'1 week'::interval);


CREATE OR REPLACE FUNCTION addAuthorization(CHARACTER VARYING(11),INTEGER,INET) RETURNS BOOLEAN AS $$
DECLARE
  inEmplid        ALIAS FOR $1;
  inUserClassId   ALIAS FOR $2;
  inRemoteAddr    ALIAS FOR $3;
  limits          RECORD;
  currentCount    RECORD;
BEGIN
  SELECT expireForEmplId(inEmplid) INTO limits;
  SELECT * INTO limits FROM byClassLimits WHERE ucId = inUserClassId;
  IF FOUND THEN
    SELECT COUNT(*) AS hostCount INTO currentCount FROM authzAddress WHERE emplid = inEmplid;
    IF FOUND AND ( currentCount.hostCount < limits.maxAddresses ) THEN
      SELECT COUNT(*) AS hostCount INTO currentCount FROM authzAddress WHERE emplid = inEmplid AND remoteAddress = inRemoteAddr;
      IF NOT FOUND OR ( currentCount.hostCount < 1 ) THEN
        INSERT INTO authzAddress
          (emplid,created,expires,remoteAddress)
          VALUES
          (
            inEmplid,
            now(),
            now() + limits.maxttl,
            inRemoteAddr
          );
        IF FOUND THEN
          RETURN TRUE;
        END IF;
      ELSE
        SELECT expires INTO currentCount FROM authzAddress WHERE emplid = inEmplid AND remoteAddress = inRemoteAddr;
        IF FOUND AND (now() >= currentCount.expires - limits.minRefresh) THEN
          UPDATE authzAddress SET expires = now() + limits.maxTTL WHERE emplid = inEmplid AND remoteAddress = inRemoteAddr;
          IF FOUND THEN
            RETURN TRUE;
          END IF;
        ELSE
          RAISE EXCEPTION 'Address % can only be refreshed after %', inRemoteAddr, currentCount.expires - limits.minRefresh;
        END IF;
      END IF;
    ELSE
      RAISE EXCEPTION 'Maximum authorized host limit reached.';
    END IF;
  ELSE
    RAISE EXCEPTION 'Invalid user class: %', inUserClassId;
  END IF;
  RETURN FALSE;
END;
$$ LANGUAGE plpgsql;

